/******************************************************************************\
 *  NOTLIB_COPY:  A function to copy an arbitrary file in block mode          *
 *  Copyright 1992 by the Caltech Odd Hack Committee.  No rights reserved     *
 *  Author:  Carl J Lydick (carl@sol1.gps.caltech.edu)                        *
 *      Arguments:                                                            *
 *              inp_file                                                      *
 *                                                                            *
 *              VMS usage: input_filespec                                     *
 *              type: character-coded text string                             *
 *              access: read only                                             *
 *              mechanism: by descriptor--fixed                               *
 *                  length string descriptor                                  *
 *                                                                            *
 *              Name of the file to be copied.                                *
 *              ---------------------------------                             *
 *              out_file                                                      *
 *                                                                            *
 *              VMS usage: output_filespec                                    *
 *              type: character-coded text string                             *
 *              access: read only                                             *
 *              mechanism: by descriptor--fixed                               *
 *                  length string descriptor                                  *
 *                                                                            *
 *              Name of the destination file.                                 *
 *              ---------------------------------                             *
 *              file_id                                                       *
 *                                                                            *
 *              VMS usage: input_file_id                                      *
 *              type: character-coded text string                             *
 *              access: read only                                             *
 *              mechansim: by descriptor--fixed                               *
 *                  length string descriptor                                  *
 *                                                                            *
 *              NAM$T_DVI, NAM$W_FID, and NAM$W_DID fields associated         *
 *              with the input file                                           *
\******************************************************************************/
#include descrip
#include rms
#include ssdef
notlib_copy(struct dsc$descriptor *inp_file, struct dsc$descriptor *out_file,
        struct dsc$descriptor *file_id)
{       struct FAB inp_fab, out_fab;
        struct RAB inp_rab, out_rab;
        struct NAM inp_nam;
        struct XABSUM xabsum;
        struct XABKEY *xabkey = 0;
        struct XABALL *xaball = 0;
        char buffer[32256];
        long stat;
        char *ptr;
        int i;

        xabsum = cc$rms_xabsum;

        inp_fab = cc$rms_fab;
        inp_fab.fab$b_fac = FAB$M_BRO | FAB$M_GET;
        inp_fab.fab$l_fna = inp_file->dsc$a_pointer;
        inp_fab.fab$b_fns = inp_file->dsc$w_length;
        inp_fab.fab$l_fop = FAB$M_SQO;
        inp_fab.fab$b_shr = FAB$M_SHRPUT | FAB$M_UPI;
        inp_fab.fab$l_xab = &xabsum;
        if (file_id != 0)
        {       inp_nam = cc$rms_nam;
                ptr = &(inp_nam.nam$t_dvi);
                for(i = 0; i < file_id->dsc$w_length; ++i)
                        ptr[i] = file_id->dsc$a_pointer[i];
                inp_fab.fab$l_fop |= FAB$M_NAM;
                inp_fab.fab$l_nam = &inp_nam;
        }
        if(((stat = SYS$OPEN(&inp_fab)) & 7) != 1)
                return stat;

        if(xabsum.xab$b_nok > 0)
        {       int i;

                if ((xabkey = (struct XABKEY *) malloc(xabsum.xab$b_nok *
                        sizeof(struct XABKEY))) == 0)
                        return SS$_INSFMEM;

                for(i = 0; i < xabsum.xab$b_nok; ++i)
                {       xabkey[i] = cc$rms_xabkey;
                        xabkey[i].xab$l_nxt = xabkey + i + 1;
                        xabkey[i].xab$b_ref = i;
                }
                xabkey[xabsum.xab$b_nok-1].xab$l_nxt = 0;
                xabsum.xab$l_nxt = xabkey;
        }
        if(xabsum.xab$b_noa > 0)
        {       int i;

                if((xaball = (struct XABALL *) malloc(xabsum.xab$b_noa *
                        sizeof(struct XABALL))) == 0)
                {       if (xabkey != 0)
                        {       free(xabkey);
                                xabkey = 0;
                        }
                        return SS$_INSFMEM;
                }
                for(i = 0; i < xabsum.xab$b_noa; ++i)
                {       xaball[i] = cc$rms_xaball;
                        xaball[i].xab$l_nxt = xaball + i + 1;
                        xaball[i].xab$b_aid = i;
                }
                xabkey[xabsum.xab$b_nok-1].xab$l_nxt = xaball;
                xaball[xabsum.xab$b_noa-1].xab$l_nxt = 0;

        }
        if(((stat = SYS$DISPLAY(&inp_fab)) & 7) != 1)
        {       if(xabkey != 0)
                {       free(xabkey);
                        xabkey = 0;
                }
                if(xaball != 0)
                {       free(xaball);
                        xaball = 0;
                }
                return stat;
        }
        out_fab = inp_fab;
        out_fab.fab$b_fac = FAB$M_BRO | FAB$M_PUT;
        out_fab.fab$l_fna = out_file->dsc$a_pointer;
        out_fab.fab$b_fns = out_file->dsc$w_length;
        out_fab.fab$w_ifi = 0;
        out_fab.fab$b_shr = FAB$M_SHRPUT | FAB$M_UPI;
        out_fab.fab$l_fop &= ~FAB$M_NAM;
        out_fab.fab$l_nam = 0;

        if(((stat = SYS$CREATE(&out_fab)) & 7) != 1)
        {       SYS$CLOSE(&inp_fab);
                if(xabkey != 0)
                {       free(xabkey);
                        xabkey = 0;
                }
                if(xaball != 0)
                {       free(xaball);
                        xaball = 0;
                }
                return stat;
        }

        inp_rab = cc$rms_rab;
        inp_rab.rab$l_fab = &inp_fab;
        inp_rab.rab$l_rop = RAB$M_BIO;

        out_rab = inp_rab;
        out_rab.rab$l_fab = &out_fab;
        out_rab.rab$w_isi = 0;

        if((((stat = SYS$CONNECT(&inp_rab)) & 7) != 1) ||
                (((stat = SYS$CONNECT(&out_rab)) & 7) != 1))
        {       SYS$CLOSE(&inp_fab);
                SYS$CLOSE(&out_fab);
                if(xabkey != 0)
                {       free(xabkey);
                        xabkey = 0;
                }
                if(xaball != 0)
                {       free(xaball);
                        xaball = 0;
                }
                return stat;
        }

        inp_rab.rab$l_ubf = buffer;
        inp_rab.rab$w_usz = 32256;
        out_rab.rab$l_rbf = buffer;

        while(1==1)
        {       if((((stat = SYS$READ(&inp_rab)) & 7) != 1) && stat != RMS$_EOF)
                {       SYS$CLOSE(&inp_fab);
                        SYS$CLOSE(&out_fab);
                        if(xabkey != 0)
                        {       free(xabkey);
                                xabkey = 0;
                        }
                        if(xaball != 0)
                        {       free(xaball);
                                xaball = 0;
                        }
                        return stat;
                }
                else if(stat == RMS$_EOF)
                {       SYS$CLOSE(&inp_fab);
                        SYS$CLOSE(&out_fab);
                        if(xabkey != 0)
                        {       free(xabkey);
                                xabkey = 0;
                        }
                        if(xaball != 0)
                        {       free(xaball);
                                xaball = 0;
                        }
                        return RMS$_NORMAL;
                }
                else
                {       out_rab.rab$w_rsz = inp_rab.rab$w_rsz;
                        if(((stat = SYS$WRITE(&out_rab)) & 7) != 1)
                        {       SYS$CLOSE(&inp_fab);
                                SYS$CLOSE(&out_fab);
                                if(xabkey != 0)
                                {       free(xabkey);
                                        xabkey = 0;
                                }
                                if(xaball != 0)
                                {       free(xaball);
                                        xaball = 0;
                                }
                                return stat;
                        }
                }
        }
}
/***************************** END of NOTLIB_COPY *****************************/
